local gen2d = require 'poisson'
local newPlanet = require 'planet'

local lg = love.graphics
local lm = love.math

local bgStars
local planets={}


local function computeStars()
	local floor = math.floor
	local dim=32
	local id = love.image.newImageData(SW,SH)
	local dist=5
	local continue=true
	while continue do
		local points = gen2d(SW,SH,dist)
		if #points > 5 then
			for _,p in ipairs(points) do
				id:setPixel(floor(p.x),floor(p.y),dim,dim,dim)
			end
		else 
			continue=false
		end
		dist=dist * 2
		dim=dim*2
		if (dim > 255) then dim = 255 end
	end
	bgStars:clear()
	lg.setCanvas(bgStars)
	lg.draw(lg.newImage(id),0,0)
	lg.setCanvas()
end


function update_load(dt)
	if loadState == "start" then
		loadState="stars"
	elseif loadState == "stars" then
		computeStars()
		loadState="planet"
		iPlanet=1
	elseif loadState == "planet" then
		planets[iPlanet]:generate()
		iPlanet=iPlanet+1
		if iPlanet > #planets then
			loadState="end"
		end
	else
		love.update = update_dyn
	end
end

function update_dyn(dt)
	for _,p in ipairs(planets) do
		p:update(dt)
	end
end

function love.load()
	math.randomseed(os.time())
	SW,SH = lg.getWidth(), lg.getHeight()

	bgStars = lg.newCanvas(SW,SH)

	local pts = gen2d(SW-100,SH-100,150)
	for _,p in ipairs(pts) do
		table.insert(planets,newPlanet(50+math.floor(p.x),50+math.floor(p.y)))
	end
	reset()
end

function reset()
	loadState="start"
	love.update = update_load
end

function love.draw()
	lg.draw(bgStars)
	for _,p in ipairs(planets) do
		p:draw()
	end
end

function love.keypressed(key)
	if key==" " or key =="return" then
		reset()
	end
end
